package cn.zgfxrc.boot.common.config.runner;

import com.github.yitter.contract.IdGeneratorOptions;
import com.github.yitter.idgen.YitIdHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

@Slf4j
@Component
@Order(Integer.MIN_VALUE)
public class SnowflakeIdGeneratorInitRunner implements ApplicationRunner {
  @Override
  public void run(ApplicationArguments args) throws Exception {
    // 创建 IdGeneratorOptions 对象，当分布式时，一定手动指定workerId避免重复，例如使用redis方案
    IdGeneratorOptions options = new IdGeneratorOptions((short) 0);
    options.WorkerIdBitLength = 10; // 默认值6，限定 WorkerId 最大值为2^6-1，即默认最多支持64个节点
    options.SeqBitLength = 10; // 默认值6，限制每毫秒生成的ID个数。若生成速度超过5万个/秒，建议加大 SeqBitLength 到 10
    // 保存参数（务必调用，否则参数设置不生效）
    YitIdHelper.setIdGenerator(options);
    // 以上过程只需全局一次，且应在生成ID之前完成。
    // 而后在需要的地方调用 YitIdHelper.nextId() 即可
    log.info("Id generator init, try gen: {}", YitIdHelper.nextId());
  }
}
