package cn.zgfxrc.boot.upms.biz.controller;

import static com.anji.captcha.model.common.RepCodeEnum.*;

import cn.zgfxrc.boot.upms.api.entity.req.SysUserReq;
import cn.zgfxrc.boot.upms.biz.service.SysUserService;
import cn.zgfxrc.boot.upms.biz.util.AesUtil;
import com.anji.captcha.model.common.ResponseModel;
import com.anji.captcha.model.vo.CaptchaVO;
import com.anji.captcha.service.CaptchaService;
import com.seepine.secret.annotation.Expose;
import com.seepine.secret.entity.AuthUser;
import com.seepine.tool.Run;
import com.seepine.tool.util.Objects;
import com.seepine.tool.util.Strings;
import com.seepine.tool.util.Validate;
import com.seepine.wrap.WrapException;
import io.swagger.annotations.Api;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2021.11.1
 */
@Slf4j
@Expose
@RestController
@Api(tags = "行业管理")
@RequestMapping("sys/api")
public class ApiController {
  @Resource private CaptchaService captchaService;
  @Resource private SysUserService sysUserService;

  @Value("${spring.profiles.active:}")
  private String active;

  //  @Log(module = "upms", title = "用户登录")
  @PostMapping("login")
  public List<AuthUser> login(@RequestBody SysUserReq entity) {
    if (!Objects.require(active, Strings.EMPTY).contains("dev")) {
      CaptchaVO captchaVO = new CaptchaVO();
      captchaVO.setCaptchaVerification(entity.getCaptchaVerification());
      ResponseModel response = captchaService.verification(captchaVO);
      if (!response.isSuccess()) {
        Validate.isFalse(
            API_CAPTCHA_INVALID.getCode().equals(response.getRepCode()),
            API_CAPTCHA_INVALID.getDesc());
        Validate.isFalse(
            API_REQ_LIMIT_VERIFY_ERROR.getCode().equals(response.getRepCode()),
            API_REQ_LIMIT_VERIFY_ERROR.getDesc());
        Validate.isFalse(
            API_REQ_INVALID.getCode().equals(response.getRepCode()), API_REQ_INVALID.getDesc());
        throw new WrapException("验证行为超时，请刷新页面重新登录");
      }
    }
    String phone = AesUtil.decrypt(entity.getPhone(), entity.getSecret());
    String password = AesUtil.decrypt(entity.getPassword(), entity.getSecret());
    log.info("phone:{},password:{}", phone, password);
    phone = Validate.nonBlank(phone, "手机号不能为空").replace(" ", "").trim();
    password = Validate.nonBlank(password, "密码不能为空").replace(" ", "").trim();
    List<AuthUser> list = new ArrayList<>();
    Run.nonEmpty(sysUserService.login(phone, password), list::addAll);
    if (Objects.isEmpty(list)) {
      throw new WrapException("账号不存在或密码错误");
    }
    return list;
  }
}
