package cn.zgfxrc.boot.upms.biz.controller;

import cn.zgfxrc.boot.common.core.entity.base.BaseModelWithTenant;
import cn.zgfxrc.boot.common.core.entity.interfaces.AddGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.DelGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.EditGroup;
import cn.zgfxrc.boot.common.core.util.TenantUtil;
import cn.zgfxrc.boot.upms.api.entity.SysDict;
import cn.zgfxrc.boot.upms.api.entity.SysDictItem;
import cn.zgfxrc.boot.upms.api.enums.DictType;
import cn.zgfxrc.boot.upms.biz.service.SysDictService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.seepine.secret.annotation.Expose;
import com.seepine.secret.annotation.Log;
import com.seepine.secret.annotation.Permission;
import com.seepine.tool.Run;
import com.seepine.tool.util.Validate;
import com.seepine.wrap.WrapException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import java.util.List;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author huanghs@zgfxrc.cn
 */
@RestController
@Api(tags = "字典管理")
@RequestMapping("sys/dict")
@AllArgsConstructor
public class SysDictController {
  SysDictService service;

  /**
   * 列表查询
   *
   * @param entity 实体类
   * @return list
   */
  @ApiOperation("获取分页")
  @PostMapping("fetch")
  @Permission("sys_dict_view")
  public List<SysDict> fetch(@RequestBody SysDict entity) {
    LambdaQueryWrapper<SysDict> wrapper = Wrappers.lambdaQuery();
    Run.nonBlankAndTrim(entity.getName(), val -> wrapper.like(SysDict::getName, val));
    wrapper.orderByAsc(SysDict::getSort);
    return service.list(wrapper);
  }

  /**
   * 新增
   *
   * @param entity 实体类
   */
  @Log("新增")
  @ApiOperation("新增")
  @PostMapping("add")
  @Permission("sys_dict_add")
  public void save(@RequestBody @Validated(AddGroup.class) SysDict entity) {
    if (!service.save(entity)) {
      throw new WrapException("保存失败");
    }
  }

  /**
   * 修改
   *
   * @param entity 实体类
   */
  @Log("修改")
  @ApiOperation("修改")
  @PostMapping("edit")
  @Permission("sys_dict_edit")
  public void updateById(@RequestBody @Validated(EditGroup.class) SysDict entity) {
    if (!service.updateById(entity)) {
      throw new WrapException("修改失败");
    }
  }

  /**
   * 通过id删除
   *
   * @param entity 实体类
   */
  @Log("删除")
  @ApiOperation("根据id删除")
  @PostMapping("del")
  @Permission("sys_dict_del")
  public void removeById(@ApiParam("主键") @RequestBody @Validated(DelGroup.class) SysDict entity) {
    if (!service.removeById(entity.getId())) {
      throw new WrapException("删除失败");
    }
  }

  /**
   * 列表查询
   *
   * @param code 字典编码
   * @return list
   */
  @ApiOperation("获取字典数据")
  @PostMapping("{code}")
  @Expose
  public List<SysDictItem> dictList(@PathVariable String code) {
    return service.dictList(code);
  }
}
