package cn.zgfxrc.boot.upms.biz.controller;

import cn.zgfxrc.boot.common.core.entity.interfaces.AddGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.DelGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.EditGroup;
import cn.zgfxrc.boot.upms.api.entity.SysDictItem;
import cn.zgfxrc.boot.upms.biz.service.SysDictItemService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.seepine.secret.annotation.Log;
import com.seepine.secret.annotation.Permission;
import com.seepine.wrap.WrapException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author huanghs@zgfxrc.cn
 */
@RestController
@Api(tags = "字典项管理")
@RequestMapping("sys/dict/item")
@AllArgsConstructor
public class SysDictItemController {
  SysDictItemService service;

  @Permission("sys_dict_view")
  @ApiOperation("获取分页")
  @PostMapping("page")
  public IPage<SysDictItem> getPageRes(Page<SysDictItem> page, @RequestBody SysDictItem entity) {
    return service.page(page, Wrappers.lambdaQuery(entity));
  }

  /**
   * 新增
   *
   * @param entity 实体类
   */
  @Log("新增")
  @ApiOperation("新增")
  @PostMapping("add")
  @Permission("sys_dict_add")
  public void save(@RequestBody @Validated(AddGroup.class) SysDictItem entity) {
    if (!service.save(entity)) {
      throw new WrapException("保存失败");
    }
  }

  /**
   * 修改
   *
   * @param entity 实体类
   */
  @Log("修改")
  @ApiOperation("修改")
  @PostMapping("edit")
  @Permission("sys_dict_edit")
  public void updateById(@RequestBody @Validated(EditGroup.class) SysDictItem entity) {
    if (!service.updateById(entity)) {
      throw new WrapException("修改失败");
    }
  }

  /**
   * 通过id删除
   *
   * @param entity 实体类
   */
  @Log("删除")
  @ApiOperation("根据id删除")
  @PostMapping("del")
  @Permission("sys_dict_del")
  public void removeById(
      @ApiParam("主键") @RequestBody @Validated(DelGroup.class) SysDictItem entity) {
    if (!service.removeById(entity.getId())) {
      throw new WrapException("删除失败");
    }
  }
}
