package cn.zgfxrc.boot.upms.biz.controller;

import cn.zgfxrc.boot.common.core.entity.interfaces.AddGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.EditGroup;
import cn.zgfxrc.boot.upms.api.entity.SysGroup;
import cn.zgfxrc.boot.upms.api.entity.SysGroupUser;
import cn.zgfxrc.boot.upms.api.entity.req.SysGroupReq;
import cn.zgfxrc.boot.upms.api.entity.res.SysGroupRes;
import cn.zgfxrc.boot.upms.api.entity.res.SysUserRes;
import cn.zgfxrc.boot.upms.api.enums.GroupUserType;
import cn.zgfxrc.boot.upms.biz.service.SysGroupService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.seepine.secret.annotation.Log;
import com.seepine.secret.annotation.Permission;
import com.seepine.tool.util.Validate;
import com.seepine.wrap.WrapException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.Collections;
import java.util.List;
import lombok.AllArgsConstructor;
import org.springframework.lang.Nullable;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author huanghs@zgfxrc.cn
 */
@RestController
@Api(tags = "用户组管理")
@RequestMapping("sys/group")
@AllArgsConstructor
public class SysGroupController {
  SysGroupService service;

  /**
   * 树查询,
   *
   * @param entity 实体类
   * @return list
   */
  @ApiOperation("获取树结构")
  @PostMapping("fetch")
  @Permission("sys_group_view")
  public List<SysGroupRes> fetch(@RequestBody @Nullable SysGroup entity) {
    return service.fetch(entity);
  }
  /**
   * 新增
   *
   * @param entity 实体类
   */
  @Log("新增")
  @ApiOperation("新增")
  @PostMapping("add")
  @Permission("sys_group_add")
  @Transactional(rollbackFor = Exception.class)
  public void save(@RequestBody @Validated(AddGroup.class) SysGroupReq entity) {
    if (!service.save(entity)) {
      throw new WrapException("保存失败");
    }
    service.resetUser(
        entity.getId(),
        Collections.singletonList(entity.getPrimaryChargeUserId()),
        GroupUserType.PRIMARY);
    service.resetUser(entity.getId(), entity.getSecondaryChargeUserIds(), GroupUserType.SECONDARY);
  }

  @Log("修改")
  @ApiOperation("修改")
  @PostMapping("edit")
  @Permission("sys_group_edit")
  @Transactional(rollbackFor = Exception.class)
  public void updateById(@RequestBody @Validated(EditGroup.class) SysGroupReq entity) {
    Validate.isTrue(service.updateById(entity), "保存失败");
    service.resetUser(
        entity.getId(),
        Collections.singletonList(entity.getPrimaryChargeUserId()),
        GroupUserType.PRIMARY);
    service.resetUser(entity.getId(), entity.getSecondaryChargeUserIds(), GroupUserType.SECONDARY);
  }

  @Log("删除")
  @ApiOperation("删除")
  @PostMapping("del")
  @Permission("sys_group_del")
  public void del(@RequestBody SysGroup entity) {
    service.del(entity);
  }

  @ApiOperation("查看组内成员")
  @PostMapping("user/page")
  public IPage<SysUserRes> userPage(Page<SysGroupUser> page, @RequestBody SysGroupUser req) {
    return service.userPage(page, req.getGroupId());
  }

  @ApiOperation("通过部门id获取用户ids")
  @PostMapping("user/ids/by/{groupId}")
  public List<Long> userIdsById(@PathVariable Long groupId) {
    return service.userIdsById(groupId);
  }

  @ApiOperation("重新设置部门成员")
  @PostMapping("user/reset")
  public void userReset(@RequestBody SysGroupReq req) {
    service.resetUser(req.getId(), req.getUserIds(), GroupUserType.NORMAL);
  }

  @ApiOperation("移除部门中某个成员")
  @PostMapping("user/remove")
  public void userRemove(@RequestBody SysGroupUser req) {
    service.userRemove(req);
  }
}
