package cn.zgfxrc.boot.upms.biz.controller;

import cn.zgfxrc.boot.common.core.entity.base.BaseModel;
import cn.zgfxrc.boot.common.core.entity.interfaces.AddGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.DelGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.EditGroup;
import cn.zgfxrc.boot.common.core.enums.PlatformType;
import cn.zgfxrc.boot.common.core.util.TreeUtil;
import cn.zgfxrc.boot.upms.api.entity.SysMenu;
import cn.zgfxrc.boot.upms.api.entity.res.SysMenuRes;
import cn.zgfxrc.boot.upms.api.entity.res.SysMenuTreeNode;
import cn.zgfxrc.boot.upms.biz.runner.MenuApplicationRunner;
import cn.zgfxrc.boot.upms.biz.service.SysMenuService;
import cn.zgfxrc.boot.upms.biz.service.SysPackageService;
import cn.zgfxrc.boot.upms.biz.service.SysRoleService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.seepine.json.Json;
import com.seepine.secret.AuthUtil;
import com.seepine.secret.annotation.Log;
import com.seepine.secret.annotation.Permission;
import com.seepine.secret.annotation.Role;
import com.seepine.tool.util.Validate;
import com.seepine.wrap.WrapException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import java.io.IOException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.lang.Nullable;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2021.11.2
 */
@RestController
@Api(tags = "菜单管理")
@RequestMapping("sys/menu")
@AllArgsConstructor
public class SysMenuController {
  private final SysRoleService sysRoleService;
  private final SysPackageService sysPackageService;
  private final SysMenuService service;

  /**
   * 分页查询
   *
   * @param page 分页对象
   * @param entity 实体类
   * @return IPage
   */
  @ApiOperation("获取分页")
  @PostMapping("page")
  @Permission("sys_menu_view")
  public IPage<SysMenu> page(Page<SysMenu> page, @RequestBody SysMenu entity) {
    LambdaQueryWrapper<SysMenu> wrapper = Wrappers.lambdaQuery(entity);
    return service.page(page, wrapper);
  }

  /**
   * 分页查询
   *
   * @param entity 实体类
   * @return List
   */
  @ApiOperation("获取数组")
  @PostMapping("list")
  @Permission("sys_menu_view")
  public List<SysMenu> list(@RequestBody @Nullable SysMenu entity) {
    LambdaQueryWrapper<SysMenu> wrapper = Wrappers.lambdaQuery(entity);
    return service.list(wrapper);
  }

  /**
   * 新增
   *
   * @param entity 实体类
   */
  @Log("新增")
  @ApiOperation("新增")
  @PostMapping("add")
  @Permission("sys_menu_add")
  public void save(@RequestBody @Validated(AddGroup.class) SysMenu entity) {
    if (!service.save(entity)) {
      throw new WrapException("保存失败");
    }
  }

  /**
   * 修改
   *
   * @param entity 实体类
   */
  @Log("修改")
  @ApiOperation("修改")
  @PostMapping("edit")
  @Permission("sys_menu_edit")
  public void updateById(@RequestBody @Validated(EditGroup.class) SysMenu entity) {
    if (!service.updateById(entity)) {
      throw new WrapException("修改失败");
    }
  }

  /**
   * 通过id删除
   *
   * @param entity 实体类
   */
  @Log("删除")
  @ApiOperation("根据id删除")
  @PostMapping("del")
  @Permission("sys_menu_del")
  public void removeById(@ApiParam("主键") @RequestBody @Validated(DelGroup.class) SysMenu entity) {
    if (!service.removeById(entity.getId())) {
      throw new WrapException("删除失败");
    }
  }

  @Permission("sys_menu_view")
  @ApiOperation("获取分页扩展")
  @PostMapping("page/res")
  public IPage<SysMenuRes> getPageRes(Page<SysMenu> page, @RequestBody SysMenu entity) {
    return service.getPageRes(page, entity);
  }

  @Permission("sys_menu_view")
  @ApiOperation("获取分页扩展")
  @PostMapping("fetch")
  public List<SysMenuTreeNode> fetch(@RequestBody SysMenu entity) {
    return service.fetch(entity);
  }

  @Permission("sys_menu_view")
  @ApiOperation("获取分页扩展")
  @PostMapping("dic/tree")
  public List<SysMenuTreeNode> dicTree(@RequestBody SysMenu entity) {
    return service.dicTree(entity);
  }

  @Permission("sys_menu_view")
  @ApiOperation("获取分页扩展")
  @PostMapping("tree/res")
  public List<SysMenuTreeNode> getPageRes(@RequestBody SysMenu entity) {
    Validate.isTrue(PlatformType.SYS.eq(AuthUtil.getUser().getClaim("platformType")), "当前用户非平台端");
    return TreeUtil.build(
        service.list(Wrappers.lambdaQuery(entity)).stream()
            .map(
                item -> {
                  SysMenuTreeNode node = new SysMenuTreeNode();
                  BeanUtils.copyProperties(item, node);
                  return node;
                })
            .toList());
  }

  @ApiOperation("获取当前登录者拥有的菜单")
  @PostMapping("tree")
  public List<SysMenuTreeNode> tree() {
    return service.tree();
  }

  @ApiOperation("获取指定角色拥有的菜单")
  @PostMapping("tree/role/{roleId}")
  public SysMenuRes treeByRole(@PathVariable Long roleId) {
    return service.treeByRoleId(roleId);
  }

  @ApiOperation("修改角色权限")
  @PostMapping("role/{roleId}")
  @Log(module = "upms", title = "修改角色权限")
  public void saveRoleMenus(@PathVariable Long roleId, @RequestBody List<Long> menuIds) {
    sysRoleService.resetRoleMenus(roleId, menuIds);
  }

  @ApiOperation("获取指定套餐拥有的菜单")
  @PostMapping("tree/package/{packageId}")
  public SysMenuRes treeByPackage(@PathVariable Long packageId) {
    return service.treeByPackageId(packageId);
  }

  @ApiOperation("修改套餐权限")
  @PostMapping("package/{packageId}")
  @Log(module = "upms", title = "修改套餐权限")
  public void savePackageMenus(@PathVariable Long packageId, @RequestBody List<Long> menuIds) {
    sysPackageService.resetMenu(packageId, menuIds);
  }

  @Role("sys")
  @ApiOperation("导出所有菜单")
  @PostMapping("export")
  @Log(module = "upms", title = "导出菜单json文件")
  public void export(HttpServletResponse response) throws IOException {
    var list =
        service
            .lambdaQuery()
            .orderByAsc(BaseModel::getCreateTime)
            .orderByAsc(SysMenu::getPlatformType)
            .list();
    var jsonStr = Json.toJson(list);
    response.setHeader(
        "Content-Disposition",
        "attachment"
            + ";filename="
            + URLEncoder.encode(MenuApplicationRunner.MENU_FILENAME, StandardCharsets.UTF_8));
    response.getOutputStream().write(jsonStr.getBytes(StandardCharsets.UTF_8));
  }
}
