package cn.zgfxrc.boot.upms.biz.controller;

import cn.zgfxrc.boot.common.core.entity.interfaces.AddGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.DelGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.EditGroup;
import cn.zgfxrc.boot.upms.api.entity.SysPosition;
import cn.zgfxrc.boot.upms.biz.service.SysPositionService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.seepine.secret.annotation.Log;
import com.seepine.secret.annotation.Permission;
import com.seepine.tool.Run;
import com.seepine.wrap.WrapException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springframework.lang.Nullable;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author linf@zgfxrc.cn
 * @date 2024.5.15
 */
@RestController
@Api(tags = "职位管理")
@RequestMapping("sys/position")
@AllArgsConstructor
public class SysPositionController {
  SysPositionService service;

  /**
   * 分页查询
   *
   * @param entity 实体类
   * @return List
   */
  @ApiOperation("获取数组")
  @PostMapping("list")
  @Permission("sys_position_view")
  public List<SysPosition> list(@RequestBody @Nullable SysPosition entity) {
    LambdaQueryWrapper<SysPosition> wrapper = Wrappers.lambdaQuery(entity);
    return service.list(wrapper);
  }

  /**
   * 新增
   *
   * @param entity 实体类
   */
  @Log("新增")
  @ApiOperation("新增")
  @PostMapping("add")
  @Permission("sys_position_add")
  public void save(@RequestBody @Validated(AddGroup.class) SysPosition entity) {
    if (!service.save(entity)) {
      throw new WrapException("保存失败");
    }
  }

  /**
   * 修改
   *
   * @param entity 实体类
   */
  @Log("修改")
  @ApiOperation("修改")
  @PostMapping("edit")
  @Permission("sys_position_edit")
  public void updateById(@RequestBody @Validated(EditGroup.class) SysPosition entity) {
    if (!service.updateById(entity)) {
      throw new WrapException("修改失败");
    }
  }

  /**
   * 通过id删除
   *
   * @param entity 实体类
   */
  @Log("删除")
  @ApiOperation("根据id删除")
  @PostMapping("del")
  @Permission("sys_position_del")
  public void removeById(@ApiParam("主键") @RequestBody @Validated(DelGroup.class) SysPosition entity) {
    if (!service.removeById(entity.getId())) {
      throw new WrapException("删除失败");
    }
  }

  /**
   * 分页查询
   *
   * @param page   分页对象
   * @param entity 实体类
   * @return IPage
   */
  @PostMapping("page")
  @Permission("sys_position_view")
  public IPage<SysPosition> page(Page<SysPosition> page, @RequestBody SysPosition entity) {
    LambdaQueryWrapper<SysPosition> wrapper = Wrappers.lambdaQuery();
    Run.nonBlankAndTrim(entity.getName(), val -> wrapper.like(SysPosition::getName, val));
    return service.page(page, wrapper);
  }

  /**
   * 列表查询
   *
   * @param entity 实体类
   * @return list
   */
  @ApiOperation("获取分页")
  @PostMapping("fetch")
  @Permission("sys_position_view")
  public List<SysPosition> fetch(@RequestBody SysPosition entity) {
    LambdaQueryWrapper<SysPosition> wrapper = Wrappers.lambdaQuery();
    Run.nonBlankAndTrim(entity.getName(), val -> wrapper.like(SysPosition::getName, val));
    return service.list(wrapper);
  }
}
