package cn.zgfxrc.boot.upms.biz.controller;

import cn.zgfxrc.boot.common.core.entity.base.BaseModel;
import cn.zgfxrc.boot.common.core.entity.interfaces.AddGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.EditGroup;
import cn.zgfxrc.boot.common.core.enums.ActiveStatus;
import cn.zgfxrc.boot.upms.api.entity.SysUser;
import cn.zgfxrc.boot.upms.api.entity.req.SysUserReq;
import cn.zgfxrc.boot.upms.api.entity.res.SysUserRes;
import cn.zgfxrc.boot.upms.biz.service.SysUserService;
import cn.zgfxrc.boot.upms.biz.util.SecureUtil;
import cn.zgfxrc.boot.upms.biz.util.ValidUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.seepine.secret.AuthUtil;
import com.seepine.secret.annotation.Log;
import com.seepine.secret.annotation.Permission;
import com.seepine.secret.entity.AuthUser;
import com.seepine.tool.util.Validate;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import java.util.Set;
import javax.validation.Valid;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2021.11.2
 */
@RestController
@Api(tags = "用户管理")
@RequestMapping("sys/user")
@AllArgsConstructor
public class SysUserController {
  private final SysUserService sysUserService;

  @Permission("sys_user_view")
  @ApiOperation("获取分页扩展")
  @PostMapping("page/res")
  public IPage<SysUserRes> getPageRes(Page<SysUser> page, @RequestBody SysUserReq entity) {
    return sysUserService.getPageRes(page, entity);
  }

  @Permission("sys_user_add")
  @PostMapping("add")
  @ApiOperation("新增用户")
  @Log(module = "upms", title = "新增用户")
  public void add(@RequestBody @Validated(AddGroup.class) SysUserReq entity) {
    sysUserService.add(entity);
  }

  @ApiOperation("修改用户信息")
  @Permission("sys_user_edit")
  @PostMapping("edit")
  @Log(module = "upms", title = "修改用户信息")
  public void updateById(@RequestBody @Validated(EditGroup.class) SysUserReq entity) {
    sysUserService.editInfo(entity);
  }

  @ApiOperation("删除用户")
  @Permission("sys_user_del")
  @PostMapping("del")
  @Log(module = "upms", title = "修改用户信息")
  public void delById(@RequestBody @Validated(EditGroup.class) SysUserReq entity) {
    sysUserService.removeById(entity);
  }
  /**
   * 个人修改密码
   *
   * @param entity {password,newPassword}
   * @apiNote cep-admin/src/api/api.ts/updatePassword
   * @date 2021.11.16
   */
  @ApiOperation("修改个人密码")
  @SneakyThrows
  @PostMapping("password")
  @Log(module = "upms", title = "修改个人密码")
  public void updatePassword(@RequestBody @Valid SysUserReq entity) {
    ValidUtil.validPassword(entity.getPassword());
    ValidUtil.validPassword(entity.getNewPassword(), "新密码");
    Validate.isFalse(entity.getPassword().equals(entity.getNewPassword()), "新密码不能与旧密码相同");
    AuthUser mime = AuthUtil.getUser();
    Validate.isTrue(
        sysUserService
            .lambdaUpdate()
            .set(SysUser::getPassword, SecureUtil.encrypt(entity.getNewPassword()))
            .eq(SysUser::getPassword, SecureUtil.encrypt(entity.getPassword()))
            .eq(BaseModel::getId, mime.getId())
            .update(),
        "旧密码错误");
  }

  @ApiOperation("获取所有用户")
  @PostMapping("dic/data")
  public List<SysUser> dicData() {
    return sysUserService
        .lambdaQuery()
        .select(BaseModel::getId, SysUser::getFullName, SysUser::getPhone, SysUser::getAvatarUrl)
        .ne(SysUser::getStatus, ActiveStatus.SUSPEND)
        .list();
  }

  @ApiOperation("获取我的权限")
  @PostMapping("get/permissions")
  public Set<String> getPermissions() {
    return AuthUtil.getPermissions();
  }
}
