package cn.zgfxrc.boot.upms.biz.feign;

import cn.zgfxrc.boot.upms.api.vo.FileInfoVo;
import cn.zgfxrc.boot.upms.biz.service.FileTemplate;
import com.seepine.secret.annotation.Expose;
import com.seepine.secret.annotation.Log;
import com.seepine.wrap.WrapException;
import com.seepine.wrap.annotation.NotWrap;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.io.IOException;
import java.util.Optional;
import javax.servlet.http.HttpServletResponse;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.Nullable;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2022.6.28
 */
@Slf4j
@NotWrap
@RestController
@Api(tags = "文件管理内部接口")
@RequestMapping("sys/file/feign")
@AllArgsConstructor
public class SysFileFeignController {
  private final FileTemplate fileTemplate;

  @Expose
  @ApiOperation("上传文件")
  @Log(module = "upms", title = "上传文件")
  @PostMapping("upload")
  public FileInfoVo upload(@RequestParam("file") @Nullable MultipartFile file) {
    if (file == null || file.isEmpty() || !StringUtils.hasText(file.getOriginalFilename())) {
      throw new WrapException("找不到上传的文件");
    }
    return fileTemplate.upload(file);
  }

  @Expose
  @ApiOperation("下传文件")
  @PostMapping("download/{filename}")
  public void download(@PathVariable String filename, HttpServletResponse response)
      throws IOException {
    fileTemplate.download(filename, response);
  }

  @Expose
  @ApiOperation("下传文件")
  @PostMapping("download/by/url")
  public void downloadByUrl(String url, HttpServletResponse response) throws IOException {
    fileTemplate.download(
        Optional.ofNullable(url).orElse("").replace("/webapi/sys/file/get/", ""), response);
  }
}
