package cn.zgfxrc.boot.upms.biz.feign;

import cn.zgfxrc.boot.common.core.entity.interfaces.AddGroup;
import cn.zgfxrc.boot.upms.api.dto.NotifyDto;
import cn.zgfxrc.boot.upms.api.entity.SysNotifyChannel;
import cn.zgfxrc.boot.upms.api.enums.NotifyChannelType;
import cn.zgfxrc.boot.upms.biz.service.SysNotifyChannelService;
import cn.zgfxrc.boot.upms.biz.util.MailUtil;
import cn.zgfxrc.boot.upms.biz.util.WorkWechatUtil;
import com.seepine.tool.util.Validate;
import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2022.3.28
 */
@Slf4j
@RestController
@Api(tags = "消息通道内部接口")
@RequestMapping("sys/notify/feign")
@AllArgsConstructor
public class SysNotifyFeignController {
  private final SysNotifyChannelService sysNotifyChannelService;

  @PostMapping("send")
  public Boolean send(@RequestBody @Validated(AddGroup.class) NotifyDto entity) {
    SysNotifyChannel find =
        Validate.nonEmpty(sysNotifyChannelService.get(entity.getType()), "未配置消息通道");
    String err;
    if (find.getType().equals(NotifyChannelType.MAIL)) {
      err =
          MailUtil.build(find.getConfig())
              .send(entity.getTo(), entity.getTitle(), entity.getContent());
    } else {
      err =
          WorkWechatUtil.build(find.getConfig())
              .send(entity.getTo(), entity.getTitle() + ":" + entity.getContent());
    }
    Validate.isBlank(err, err);
    return true;
  }
}
