package cn.zgfxrc.boot.upms.biz.feign;

import cn.zgfxrc.boot.upms.api.entity.SysTenant;
import cn.zgfxrc.boot.upms.api.entity.req.SysTenantReq;
import cn.zgfxrc.boot.upms.api.entity.res.SysTenantRes;
import cn.zgfxrc.boot.upms.biz.service.SysTenantService;
import com.seepine.secret.annotation.Expose;
import com.seepine.secret.annotation.Log;
import com.seepine.tool.util.Validate;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2022.3.28
 */
@Slf4j
@RestController
@Api(tags = "租户管理内部接口")
@RequestMapping("sys/tenant/feign")
@AllArgsConstructor
public class SysTenantFeignController {
  private final SysTenantService sysTenantService;

  @ApiOperation("新增租户")
  @PostMapping("add")
  @Log(module = "upms", title = "新增租户")
  public SysTenantRes add(@RequestBody SysTenantReq entity) {
    return sysTenantService.add(entity, entity.getPlatformType());
  }

  @ApiOperation("编辑租户")
  @PostMapping("edit")
  @Log(module = "upms", title = "编辑租户")
  public Boolean edit(@RequestBody SysTenantReq entity) {
    sysTenantService.updateTenant(entity);
    return true;
  }

  @ApiOperation("删除租户")
  @PostMapping("del/{id}")
  @Log(module = "upms", title = "删除租户")
  public void remove(@PathVariable Long id) {
    Validate.isTrue(sysTenantService.removeById(id), "删除失败");
  }

  @ApiOperation("修改租户名称")
  @PostMapping("update/name")
  @Expose
  @Log(module = "upms", title = "修改租户名称")
  public Boolean updateName(
      @RequestParam Long id, @RequestParam String name, @RequestParam String shortName) {
    try {
      sysTenantService.updateName(id, name, shortName);
      return true;
    } catch (Exception e) {
      log.error("修改租户名称失败:{}", e.getMessage());
      return false;
    }
  }

  @Expose
  @ApiOperation("获取租户信息")
  @PostMapping("get/{id}")
  @Log(module = "upms", title = "修改租户名称")
  public SysTenant getById(@PathVariable Long id) {
    return sysTenantService.getById(id);
  }
}
