package cn.zgfxrc.boot.upms.biz.feign;

import cn.zgfxrc.boot.common.core.util.TenantUtil;
import cn.zgfxrc.boot.upms.api.entity.SysUser;
import cn.zgfxrc.boot.upms.biz.service.SysMenuService;
import cn.zgfxrc.boot.upms.biz.service.SysUserService;
import com.seepine.secret.AuthUtil;
import com.seepine.secret.annotation.Expose;
import com.seepine.secret.entity.AuthUser;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

/**
 * @author huanghs
 */
@RestController
@Api(tags = "租户管理内部接口")
@RequestMapping("sys/user/feign")
@AllArgsConstructor
public class SysUserFeignController {
  private final SysUserService sysUserService;
  private final SysMenuService sysMenuService;

  @ApiOperation("获取用户")
  @PostMapping("get")
  public SysUser get(@RequestParam Long id) {
    return sysUserService.getById(id);
  }

  @ApiOperation("获取用户忽略租户")
  @PostMapping("get/ignore/tenant")
  public SysUser getIgnoreTenant(@RequestParam Long id) {
    return sysUserService.getBaseMapper().getByIdWithoutTenantId(id);
  }

  @Expose(skip = true)
  @ApiOperation("获取用户权限集合")
  @PostMapping("get/permissions")
  public Set<String> getPermissions(@RequestBody AuthUser authUser) {
    return TenantUtil.apply(
        authUser.getTenantId(),
        () ->
            AuthUtil.execute(
                authUser,
                () -> {
                  List<String> permissions = sysMenuService.getPermissions(AuthUtil.getUser());
                  return new HashSet<>(permissions);
                }));
  }
}
