package cn.zgfxrc.boot.upms.biz.runner;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.zgfxrc.boot.upms.api.entity.SysMenu;
import cn.zgfxrc.boot.upms.biz.service.SysMenuService;
import com.fasterxml.jackson.core.type.TypeReference;
import com.seepine.json.Json;
import com.seepine.tool.cache.Cache;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.List;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Component;

@Slf4j
@Component
@Order(Integer.MIN_VALUE + 10)
@AllArgsConstructor
public class MenuApplicationRunner implements ApplicationRunner {
  private final SysMenuService sysMenuService;
  private static final String CACHE_KEY = "menu_json_refactor_last_modified_time";
  public static final String MENU_FILENAME = "menu.json";

  @Override
  public void run(ApplicationArguments args) throws Exception {
    Resource rs = new ClassPathResource(MENU_FILENAME);
    if (!rs.exists()) {
      log.info("未检测到upms/resources/{}文件，不进行菜单数据处理", MENU_FILENAME);
      return;
    }
    Long last = Cache.get(CACHE_KEY);
    long lastModified = rs.lastModified();
    if (last != null && last == lastModified) {
      log.info(
          "检测到upms/resources/{}文件但没有更新，上次更新时间：{}",
          MENU_FILENAME,
          DateUtil.format(new Date(lastModified), DatePattern.NORM_DATETIME_PATTERN));
      return;
    }
    var jsonStr = new String(rs.getInputStream().readAllBytes(), StandardCharsets.UTF_8);
    List<SysMenu> list;
    try {
      list = Json.parse(jsonStr, new TypeReference<>() {});
    } catch (Exception e) {
      log.error("转换 {} 数据失败，请确认内容是否为jsonArray格式的json文件", MENU_FILENAME);
      return;
    }
    log.info("读取到数据共：{}", list.size());
    log.info("清理sys_menu数据共：{}", sysMenuService.getBaseMapper().deleteAll());
    sysMenuService.removeGlobalCache();
    log.info("完成sys_menu数据插入共：{}", sysMenuService.getBaseMapper().insertBatchSomeColumn(list));
    Cache.set(CACHE_KEY, lastModified);
  }
}
