package cn.zgfxrc.boot.upms.biz.service;

import cn.zgfxrc.boot.common.core.enums.DistrictType;
import cn.zgfxrc.boot.upms.api.entity.SysDistrict;
import cn.zgfxrc.boot.upms.api.entity.res.SysDistrictTreeNode;
import cn.zgfxrc.boot.upms.biz.mapper.SysDistrictMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.seepine.tool.cache.Cache;
import com.seepine.tool.util.Trees;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.function.Consumer;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2023.2.10
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysDistrictService extends ServiceImpl<SysDistrictMapper, SysDistrict> {
  private static final String DISTRICT_TREE_CACHE_KEY = "sys_district:tree";

  /**
   * 获取省市区树结构
   *
   * @return tree list
   */
  public List<SysDistrictTreeNode> tree() {
    //    return Cache.getIfPresent(
    //        DISTRICT_TREE_CACHE_KEY,
    //        () -> buildTree(listParallel(null)),
    //        Duration.ofDays(1).toMillis());
    return buildTree(listParallel(null));
  }

  /**
   * 获取省市树结构,不包含区
   *
   * @return tree list
   */
  public List<SysDistrictTreeNode> treeCity() {
    //    return Cache.getIfPresent(
    //        DISTRICT_TREE_CACHE_KEY + ":city",
    //        () -> buildTree(listParallel(w -> w.ne(SysDistrict::getType, DistrictType.DISTRICT))),
    //        Duration.ofDays(1).toMillis());
    return buildTree(listParallel(w -> w.ne(SysDistrict::getType, DistrictType.DISTRICT)));
  }

  public List<SysDistrict> listParallel(Consumer<LambdaQueryWrapper<SysDistrict>> wrapperConsumer) {
    List<CompletableFuture<List<SysDistrict>>> tasks = new ArrayList<>();
    for (int i = 0; i < 8; i++) {
      int finalI = i;
      tasks.add(
          CompletableFuture.supplyAsync(
              () -> {
                if (wrapperConsumer == null) {
                  //                  return lambdaQuery().page(Page.of(finalI + 1,
                  // 1000)).getRecords();
                  return Cache.getIfPresent(
                      DISTRICT_TREE_CACHE_KEY + ":page:" + finalI,
                      () -> lambdaQuery().page(Page.of(finalI + 1, 1000)).getRecords(),
                      Duration.ofDays(3).toMillis());
                }
                //                return lambdaQuery()
                //                    .and(wrapperConsumer)
                //                    .page(Page.of(finalI + 1, 1000))
                //                    .getRecords();
                return Cache.getIfPresent(
                    DISTRICT_TREE_CACHE_KEY + ":only_city:page:" + finalI,
                    () ->
                        lambdaQuery()
                            .and(wrapperConsumer)
                            .page(Page.of(finalI + 1, 1000))
                            .getRecords(),
                    Duration.ofDays(3).toMillis());
              }));
    }
    CompletableFuture.allOf(tasks.toArray(CompletableFuture[]::new)).join();
    List<SysDistrict> res = new ArrayList<>();
    for (CompletableFuture<List<SysDistrict>> task : tasks) {
      try {
        res.addAll(task.get(1, TimeUnit.MINUTES));
      } catch (InterruptedException | TimeoutException | ExecutionException e) {
        throw new RuntimeException(e);
      }
    }
    return res;
  }

  /** 清除缓存 */
  public void clearCache() {
    Cache.removeByPattern(DISTRICT_TREE_CACHE_KEY + "*");
  }

  private List<SysDistrictTreeNode> buildTree(List<SysDistrict> list) {
    return Trees.build(
        list.parallelStream()
            .map(
                item -> {
                  var node = new SysDistrictTreeNode();
                  BeanUtils.copyProperties(item, node);
                  node.setSort(Integer.valueOf(item.getId()));
                  return node;
                })
            .toList(),
        SysDistrictTreeNode::getId,
        SysDistrictTreeNode::getParentId,
        "0",
        SysDistrictTreeNode::getSort,
        SysDistrictTreeNode::setChildren);
  }
}
