package cn.zgfxrc.boot.upms.biz.service;

import cn.zgfxrc.boot.common.core.entity.base.BaseModel;
import cn.zgfxrc.boot.common.core.util.TenantUtil;
import cn.zgfxrc.boot.upms.api.entity.SysNotifyChannel;
import cn.zgfxrc.boot.upms.api.enums.NotifyChannelType;
import cn.zgfxrc.boot.upms.biz.mapper.SysNotifyChannelMapper;
import cn.zgfxrc.boot.upms.biz.util.MailUtil;
import cn.zgfxrc.boot.upms.biz.util.SecureUtil;
import cn.zgfxrc.boot.upms.biz.util.WorkWechatUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.seepine.secret.AuthUtil;
import com.seepine.tool.Run;
import com.seepine.tool.lock.Lock;
import com.seepine.tool.util.Validate;
import lombok.AllArgsConstructor;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

/**
 * @author huanghs
 * @date 2022.7.25
 */
@Service
@AllArgsConstructor
public class SysNotifyChannelService extends ServiceImpl<SysNotifyChannelMapper, SysNotifyChannel> {
  private static final String SYNC_KEY = "sys_notify_channel:";

  public void edit(SysNotifyChannel entity) {
    Lock.sync(
        SYNC_KEY + entity.getType().value + TenantUtil.getTenantId(),
        () -> {
          Run.nonBlankAndTrim(
              entity.getConfig(),
              val -> {
                // 测试消息发送
                if (entity.getType().equals(NotifyChannelType.MAIL)) {
                  String err =
                      MailUtil.build(entity.getConfig())
                          .send(
                              null,
                              "【FxBoot】邮件测试",
                              "这是测试内容，支持<span style='color:red'>html</span>内容");
                  Validate.isBlank(err, "保存失败，" + err);
                } else if (entity.getType().equals(NotifyChannelType.WE_CHAT)) {
                  String err =
                      WorkWechatUtil.build(entity.getConfig())
                          .send(
                              Validate.nonBlank(AuthUtil.getUser().getEmail(), "请先设置当前邮箱"),
                              "配置企业微信消息通知成功！请注意平台用户的邮箱信息需与企业微信的邮箱一致才可收到消息");
                  Validate.isBlank(err, "保存失败，" + err);
                }
                // 加密保存到库
                if (StringUtils.hasText(val)) {
                  entity.setConfig(SecureUtil.encodeAes(val));
                }
              });
          SysNotifyChannel find = get(entity.getType());
          if (find == null) {
            Validate.isTrue(super.save(entity), "保存失败");
          } else {
            if (StringUtils.hasText(entity.getConfig())) {
              Validate.isTrue(
                  super.lambdaUpdate()
                      .eq(BaseModel::getId, find.getId())
                      .set(SysNotifyChannel::getConfig, entity.getConfig())
                      .update(),
                  "保存失败");
            } else {
              Validate.isTrue(super.removeById(find.getId()), "删除失败");
            }
          }
        });
  }

  /**
   * 根据类型获取消息通道配置
   *
   * @param type 通道类型
   * @return 返回解密的config对象
   */
  @Nullable
  public SysNotifyChannel get(@NonNull NotifyChannelType type) {
    SysNotifyChannel find = lambdaQuery().eq(SysNotifyChannel::getType, type).last("limit 1").one();
    Run.nonEmpty(
        find,
        val ->
            Run.nonBlank(
                val.getConfig(),
                (config) -> find.setConfig(SecureUtil.decodeAes(val.getConfig()))));
    return find;
  }
}
