package cn.zgfxrc.boot.upms.biz.service;

import cn.zgfxrc.boot.upms.api.entity.SysPackage;
import cn.zgfxrc.boot.upms.api.entity.SysPackageMenu;
import cn.zgfxrc.boot.upms.api.entity.SysTenant;
import cn.zgfxrc.boot.upms.biz.mapper.SysPackageMapper;
import cn.zgfxrc.boot.upms.biz.mapper.SysPackageMenuMapper;
import cn.zgfxrc.boot.upms.biz.mapper.SysTenantMapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.seepine.tool.util.ListUtil;
import com.seepine.tool.util.Objects;
import com.seepine.wrap.WrapException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2021.11.2
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysPackageService extends ServiceImpl<SysPackageMapper, SysPackage> {
  private final SysPackageMenuMapper sysPackageMenuMapper;
  private final SysTenantMapper sysTenantMapper;

  /**
   * 通过租户id，获取有哪些菜单权限
   *
   * @param tenantId tenantId
   * @return menuIds
   */
  @SneakyThrows
  public List<Long> getMenuIdsByTenantId(Long tenantId) {
    SysTenant find = sysTenantMapper.selectById(tenantId);
    if (find == null) {
      throw new WrapException("获取租户错误");
    }
    return getMenuIds(Arrays.asList(find.getPackageIds()));
  }
  /**
   * 根据packageIds 获取关联的menuIds
   *
   * @param packageIds ids
   * @return menuIds
   * @date 2021.11.5
   */
  public List<Long> getMenuIds(List<Long> packageIds) {
    if (Objects.isEmpty(packageIds)) {
      return new ArrayList<>();
    }
    return ListUtil.map(
        sysPackageMenuMapper.selectList(
            Wrappers.<SysPackageMenu>lambdaQuery().in(SysPackageMenu::getPackageId, packageIds)),
        SysPackageMenu::getMenuId);
  }
  /**
   * 重置套餐菜单数据
   *
   * @param packageId 角色id
   * @param menuIds 菜单ids
   * @date 2021.11.9 huanghs@zgfxrc.cn
   */
  @Transactional(rollbackFor = Exception.class)
  public void resetMenu(Long packageId, List<Long> menuIds) {
    removeMenus(packageId);
    save(packageId, menuIds);
  }

  /**
   * 保存套餐和菜单关联表
   *
   * @param packageId packageId
   * @param menuIds menuIds
   * @date 2021.11.9 huanghs@zgfxrc.cn
   */
  @SneakyThrows
  public void save(Long packageId, List<Long> menuIds) {
    List<SysPackageMenu> list =
        ListUtil.map(
            menuIds,
            menuId -> {
              SysPackageMenu roleMenu = new SysPackageMenu();
              roleMenu.setPackageId(packageId);
              roleMenu.setMenuId(menuId);
              return roleMenu;
            });
    if (!Objects.isEmpty(list)) {
      if (sysPackageMenuMapper.insertBatchSomeColumn(list) <= 0) {
        throw new WrapException("保存失败");
      }
    }
  }
  /**
   * 移除套餐所有菜单
   *
   * @param packageId 套餐id
   * @date 2021.11.9 huanghs@zgfxrc.cn
   */
  public void removeMenus(Long packageId) {
    sysPackageMenuMapper.delete(
        Wrappers.<SysPackageMenu>lambdaQuery().eq(SysPackageMenu::getPackageId, packageId));
  }
}
