package cn.zgfxrc.boot.upms.biz.util;

import cn.hutool.core.lang.Validator;
import cn.hutool.extra.mail.Mail;
import cn.hutool.extra.mail.MailAccount;
import com.fasterxml.jackson.databind.JsonNode;
import com.seepine.json.Json;
import com.seepine.tool.util.Validate;
import com.sun.mail.util.MailSSLSocketFactory;
import lombok.extern.slf4j.Slf4j;

/**
 * @author huanghs
 * @date 2022.7.27
 */
@Slf4j
public class MailUtil {
  public static MailBuilder build(String config) {
    return new MailBuilder(config);
  }

  public static class MailBuilder {
    Mail mail;
    MailAccount mailAccount;

    public MailBuilder(String config) {
      try {
        JsonNode jsonNode = Json.parse(config);
        mailAccount = new MailAccount();
        mailAccount.setAuth(true);
        mailAccount.setSslEnable(jsonNode.get("ssl").asBoolean());
        mailAccount.setHost(jsonNode.get("host").asText());
        mailAccount.setPort(jsonNode.get("port").asInt());
        mailAccount.setFrom(jsonNode.get("from").asText());
        mailAccount.setUser(jsonNode.get("user").asText());
        mailAccount.setPass(jsonNode.get("pass").asText());
        Validate.isTrue(Validator.isEmail(mailAccount.getFrom()), "发件人邮箱格式错误");
        // 忽略ssl校验
        MailSSLSocketFactory sf = new MailSSLSocketFactory();
        sf.setTrustAllHosts(true);
        mailAccount.setCustomProperty("mail.smtp.ssl.socketFactory", sf);

        mail = Mail.create(mailAccount);
      } catch (Exception e) {
        log.error("邮件初始化失败:" + e.getMessage());
      }
    }

    public String send(String to, String title, String content) {
      try {
        if (mail == null) {
          return "邮件初始化失败";
        }
        mail.setTos(to == null ? mailAccount.getFrom() : to)
            .setTitle(title)
            .setContent("<pre style=\"white-space: pre-wrap;\">" + content + "</pre>")
            .setHtml(true)
            .send();
        return null;
      } catch (Exception e) {
        log.error("发送邮件失败:" + e.getMessage());
        return "发送邮件失败:" + e.getMessage();
      }
    }
  }
}
