package cn.zgfxrc.boot.upms.biz.util;

import com.seepine.http.Request;
import com.seepine.json.Json;
import com.seepine.json.JsonObject;
import com.seepine.tool.cache.Cache;
import com.seepine.tool.util.Validate;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class WorkWechatUtil {
  public static final String SEND_URL = "https://qyapi.weixin.qq.com/cgi-bin/message/send";

  public static WorkWechatBuilder build(String config) {
    return new WorkWechatBuilder(config);
  }

  public static class WorkWechatBuilder {
    String corpId;
    String agentId;
    String corpSecret;

    public WorkWechatBuilder(String config) {
      try {
        JsonObject jsonNode = Json.parseObj(config);
        corpId = jsonNode.getStr("corpid");
        agentId = jsonNode.getStr("agentId");
        corpSecret = jsonNode.getStr("corpsecret");
      } catch (Exception e) {
        log.error("企业微信参数获取失败:" + e.getMessage());
      }
    }

    public String getAccessToken() {
      final String KEY = "work_wechat:" + corpId + ":" + corpSecret;
      String token = Cache.getStr(KEY);
      if (token == null) {
        AccessBody res =
            Json.parse(
                Request.get(
                        "https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid="
                            + corpId
                            + "&corpsecret="
                            + corpSecret)
                    .execute()
                    .bodyStr(),
                AccessBody.class);
        Validate.isTrue(res.errcode == 0, "获取企业微信授权失败：" + res.errmsg);
        token = res.access_token;
        Cache.set(KEY, token, res.expires_in * 1000);
      }
      return token;
    }

    public String send(String toUserId, String content) {
      try {
        AccessBody res =
            Json.parse(
                Request.post(SEND_URL + "?access_token=" + getAccessToken())
                    .body(
                        new JsonObject()
                            .set("touser", toUserId)
                            .set("msgtype", "text")
                            .set("agentid", agentId)
                            .set("text", new JsonObject().set("content", content))
                            // 10s 之内不能发送重复信息
                            .set("enable_duplicate_check", 1)
                            .set("duplicate_check_interval", 10)
                            .toString())
                    .execute()
                    .bodyStr(),
                AccessBody.class);
        Validate.isTrue(res.errcode == 0, "发送企业微信消息失败：" + res.errmsg);
        return null;
      } catch (Exception e) {
        return e.getMessage();
      }
    }
  }

  @Data
  public static class AccessBody {
    Integer errcode;
    String errmsg;
    String access_token;
    Integer expires_in;
  }
}
